@@ -26,13 +26,14 @@ import numpy as np
 import geopandas as gpd
 import shapely
 import pandas as pd
+import affine
 
 
 # Testing sample_from_array
 ###########################################################
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_from_array(dem):
     from gemgis.raster import sample_from_array
@@ -52,11 +53,11 @@ def test_sample_from_array(dem):
 
 @pytest.mark.parametrize("gdf",
                          [
-                             gpd.read_file('../../gemgis/data/Test1/interfaces1.shp')
+                             gpd.read_file('../../gemgis/tests/data/interfaces1.shp')
                          ])
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_from_array(gdf, dem):
     from gemgis.raster import sample_from_array
@@ -82,7 +83,7 @@ def test_sample_from_array(gdf, dem):
 
 @pytest.mark.parametrize("array",
                          [
-                             np.load('../../gemgis/data/Test1/array_rbf.npy')
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
                          ])
 def test_sample(array):
     from gemgis.raster import sample_from_array
@@ -96,7 +97,7 @@ def test_sample(array):
 
 @pytest.mark.parametrize("array",
                          [
-                             np.load('../../gemgis/data/Test1/array_rbf.npy')
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
                          ])
 def test_sample_error(array):
     from gemgis.raster import sample_from_array
@@ -130,11 +131,11 @@ def test_sample_error(array):
 ###########################################################
 @pytest.mark.parametrize("gdf",
                          [
-                             gpd.read_file('../../gemgis/data/Test1/interfaces1.shp')
+                             gpd.read_file('../../gemgis/tests/data/interfaces1.shp')
                          ])
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_from_rasterio(gdf, dem):
     from gemgis.raster import sample_from_rasterio
@@ -158,11 +159,11 @@ def test_sample_from_rasterio(gdf, dem):
 
 @pytest.mark.parametrize("gdf",
                          [
-                             gpd.read_file('../../gemgis/data/Test1/interfaces1.shp')
+                             gpd.read_file('../../gemgis/tests/data/interfaces1.shp')
                          ])
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_from_rasterio2(gdf, dem):
     from gemgis.raster import sample_from_rasterio
@@ -183,7 +184,7 @@ def test_sample_from_rasterio2(gdf, dem):
 
 @pytest.mark.parametrize("array",
                          [
-                             np.load('../../gemgis/data/Test1/array_rbf.npy')
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
                          ])
 def test_sample_randomly_go(array):
     from gemgis.raster import sample_randomly
@@ -193,13 +194,13 @@ def test_sample_randomly_go(array):
     assert array.shape == (1069, 972)
     assert isinstance(random_sample[0], float)
     assert isinstance(random_sample[1], list)
-    assert random_sample == (518.1631561814993,  [1445.7965230270515, 1700.1554076257776])
+    assert random_sample == (518.1631561814993, [1445.7965230270515, 1700.1554076257776])
     assert all(isinstance(n, float) for n in random_sample[1])
 
 
 @pytest.mark.parametrize("array",
                          [
-                             np.load('../../gemgis/data/Test1/array_rbf.npy')
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
                          ])
 def test_sample_randomly_error(array):
     from gemgis.raster import sample_randomly
@@ -215,7 +216,7 @@ def test_sample_randomly_error(array):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_randomly_array(dem):
     from gemgis.raster import sample_randomly
@@ -232,7 +233,7 @@ def test_sample_randomly_array(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_randomly_array_multiple(dem):
     from gemgis.raster import sample_randomly
@@ -257,7 +258,7 @@ def test_sample_randomly_array_multiple(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_randomly_raster(dem):
     from gemgis.raster import sample_randomly
@@ -274,7 +275,7 @@ def test_sample_randomly_raster(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_randomly_raster_multiple(dem):
     from gemgis.raster import sample_randomly
@@ -301,7 +302,7 @@ def test_sample_randomly_raster_multiple(dem):
 ###########################################################
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_calculate_hillshades_array(dem):
     from gemgis.raster import calculate_hillshades
@@ -319,7 +320,7 @@ def test_calculate_hillshades_array(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             np.load('../../gemgis/data/Test1/array_rbf.npy')
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
                          ])
 def test_calculate_hillshades_array2(dem):
     from gemgis.raster import calculate_hillshades
@@ -336,7 +337,7 @@ def test_calculate_hillshades_array2(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_calculate_hillshades_raster(dem):
     from gemgis.raster import calculate_hillshades
@@ -354,7 +355,7 @@ def test_calculate_hillshades_raster(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_calculate_hillshades_error(dem):
     from gemgis.raster import calculate_hillshades
@@ -385,7 +386,7 @@ def test_calculate_hillshades_error(dem):
 ###########################################################
 @pytest.mark.parametrize("raster",
                          [
-                             rasterio.open('../../gemgis/tests/data/test_raster.tif')
+                             rasterio.open('../../gemgis_data/data/tests/test_raster.tif')
                          ])
 def test_calculate_slope(raster):
     from gemgis.raster import calculate_slope
@@ -406,7 +407,7 @@ def test_calculate_slope(raster):
 
 @pytest.mark.parametrize("array",
                          [
-                             np.load('../../gemgis/data/Test1/array_rbf.npy')
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
                          ])
 def test_calculate_slope_array(array):
     from gemgis.raster import calculate_slope
@@ -424,7 +425,7 @@ def test_calculate_slope_array(array):
 
 @pytest.mark.parametrize("raster",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_calculate_slope_raster(raster):
     from gemgis.raster import calculate_slope
@@ -441,7 +442,7 @@ def test_calculate_slope_raster(raster):
 
 @pytest.mark.parametrize("raster",
                          [
-                             rasterio.open('../../gemgis/tests/data/test_raster.tif')
+                             rasterio.open('../../gemgis_data/data/tests/test_raster.tif')
                          ])
 def test_calculate_slope(raster):
     from gemgis.raster import calculate_slope
@@ -454,7 +455,7 @@ def test_calculate_slope(raster):
 ###########################################################
 @pytest.mark.parametrize("raster",
                          [
-                             rasterio.open('../../gemgis/tests/data/test_raster.tif')
+                             rasterio.open('../../gemgis_data/data/tests/test_raster.tif')
                          ])
 def test_calculate_aspect(raster):
     from gemgis.raster import calculate_aspect
@@ -475,7 +476,7 @@ def test_calculate_aspect(raster):
 
 @pytest.mark.parametrize("raster",
                          [
-                             np.load('../../gemgis/data/Test1/array_rbf.npy')
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
                          ])
 def test_calculate_aspect_array(raster):
     from gemgis.raster import calculate_aspect
@@ -493,7 +494,7 @@ def test_calculate_aspect_array(raster):
 
 @pytest.mark.parametrize("raster",
                          [
-                             rasterio.open('../../gemgis/tests/data/test_raster.tif')
+                             rasterio.open('../../gemgis_data/data/tests/test_raster.tif')
                          ])
 def test_calculate_aspect_error(raster):
     from gemgis.raster import calculate_aspect
@@ -506,7 +507,7 @@ def test_calculate_aspect_error(raster):
 ###########################################################
 @pytest.mark.parametrize("raster",
                          [
-                             rasterio.open('../../gemgis/tests/data/test_raster.tif')
+                             rasterio.open('../../gemgis_data/data/tests/test_raster.tif')
                          ])
 def test_clip_by_shape(raster):
     from gemgis.raster import clip_by_polygon
@@ -526,7 +527,7 @@ def test_clip_by_shape(raster):
 
 @pytest.mark.parametrize("raster",
                          [
-                             np.load('../../gemgis/data/Test1/array_rbf.npy')
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
                          ])
 def test_clip_by_shape_array(raster):
     from gemgis.raster import clip_by_polygon
@@ -547,7 +548,7 @@ def test_clip_by_shape_array(raster):
 
 @pytest.mark.parametrize("raster",
                          [
-                             rasterio.open('../../gemgis/tests/data/test_raster.tif')
+                             rasterio.open('../../gemgis_data/data/tests/test_raster.tif')
                          ])
 @pytest.mark.parametrize("shape",
                          [
@@ -568,11 +569,10 @@ def test_clip_by_shape_error(raster, shape):
 ###########################################################
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_orientations_from_raster(dem):
     from gemgis.raster import sample_orientations
-    from gemgis.utils import set_extent
 
     orientations = sample_orientations(raster=dem,
                                        random_samples=5,
@@ -585,7 +585,7 @@ def test_sample_orientations_from_raster(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_orientations_from_raster_point(dem):
     from gemgis.raster import sample_orientations
@@ -606,7 +606,7 @@ def test_sample_orientations_from_raster_point(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_orientations_from_raster_points(dem):
     from gemgis.raster import sample_orientations
@@ -627,7 +627,7 @@ def test_sample_orientations_from_raster_points(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_orientations_from_raster_points3(dem):
     from gemgis.raster import sample_orientations
@@ -648,7 +648,7 @@ def test_sample_orientations_from_raster_points3(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_orientations_from_raster_error(dem):
     from gemgis.raster import sample_orientations
@@ -688,7 +688,7 @@ def test_sample_orientations_from_raster_error(dem):
 ###########################################################
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_interfaces_from_raster(dem):
     from gemgis.raster import sample_interfaces
@@ -704,7 +704,7 @@ def test_sample_interfaces_from_raster(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_interfaces_from_raster_point(dem):
     from gemgis.raster import sample_interfaces
@@ -725,7 +725,7 @@ def test_sample_interfaces_from_raster_point(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_interfaces_from_raster_points(dem):
     from gemgis.raster import sample_interfaces
@@ -746,7 +746,7 @@ def test_sample_interfaces_from_raster_points(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_interfaces_from_raster_points3(dem):
     from gemgis.raster import sample_interfaces
@@ -767,7 +767,7 @@ def test_sample_interfaces_from_raster_points3(dem):
 
 @pytest.mark.parametrize("dem",
                          [
-                             rasterio.open('../../gemgis/data/Test1/raster1.tif')
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
                          ])
 def test_sample_interfaces_from_raster_error(dem):
     from gemgis.raster import sample_interfaces
@@ -803,3 +803,285 @@ def test_sample_interfaces_from_raster_error(dem):
         sample_interfaces(raster=[dem],
                           extent=extent,
                           formation='surface')
+
+
+# Testing resize_raster_by_array
+###########################################################
+@pytest.mark.parametrize("array1",
+                         [
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
+                         ])
+@pytest.mark.parametrize("array2",
+                         [
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
+                         ])
+def test_resize_by_array(array1, array2):
+    from gemgis.raster import resize_by_array
+
+    array_rescaled = resize_by_array(raster=array1,
+                                     array=array2)
+
+    assert array1.read(1).ndim == 2
+    assert array1.read(1).shape == (275, 250)
+
+    assert array2.ndim == 2
+    assert array2.shape == (1069, 972)
+
+    assert array_rescaled.ndim == 2
+    assert array_rescaled.shape == (1069, 972)
+
+
+@pytest.mark.parametrize("array2",
+                         [
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
+                         ])
+@pytest.mark.parametrize("array1",
+                         [
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
+                         ])
+def test_resize_by_array_2(array1, array2):
+    from gemgis.raster import resize_by_array
+
+    array_rescaled = resize_by_array(raster=array1,
+                                     array=array2)
+
+    assert array2.read(1).ndim == 2
+    assert array2.read(1).shape == (275, 250)
+
+    assert array1.ndim == 2
+    assert array1.shape == (1069, 972)
+
+    assert array_rescaled.ndim == 2
+    assert array_rescaled.shape == (275, 250)
+
+
+@pytest.mark.parametrize("array1",
+                         [
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
+                         ])
+@pytest.mark.parametrize("array2",
+                         [
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
+                         ])
+def test_resize_by_array_error(array1, array2):
+    from gemgis.raster import resize_by_array
+
+    with pytest.raises(TypeError):
+        resize_by_array(raster=[array1],
+                        array=array2)
+
+    with pytest.raises(TypeError):
+        resize_by_array(raster=array1.read(1),
+                        array=[array2])
+
+
+# Testing resize_raster
+###########################################################
+@pytest.mark.parametrize("array1",
+                         [
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
+                         ])
+def test_resize_raster(array1):
+    from gemgis.raster import resize_raster
+
+    array_rescaled = resize_raster(raster=array1,
+                                   width=500,
+                                   height=500)
+
+    assert array1.read(1).ndim == 2
+    assert array1.read(1).shape == (275, 250)
+
+    assert array_rescaled.ndim == 2
+    assert array_rescaled.shape == (500, 500)
+
+
+@pytest.mark.parametrize("array1",
+                         [
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
+                         ])
+def test_resize_raster_array(array1):
+    from gemgis.raster import resize_raster
+
+    array_rescaled = resize_raster(raster=array1,
+                                   width=500,
+                                   height=500)
+
+    assert array1.ndim == 2
+    assert array1.shape == (1069, 972)
+
+    assert array_rescaled.ndim == 2
+    assert array_rescaled.shape == (500, 500)
+
+
+@pytest.mark.parametrize("array1",
+                         [
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
+                         ])
+@pytest.mark.parametrize("array2",
+                         [
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
+                         ])
+def test_rescale_raster_error(array1, array2):
+    from gemgis.raster import resize_raster
+
+    with pytest.raises(TypeError):
+        resize_raster(raster=[array1.read(1)],
+                      width=500,
+                      height=500)
+
+    with pytest.raises(TypeError):
+        resize_raster(raster=array1.read(1),
+                      width='500',
+                      height='500')
+
+    with pytest.raises(TypeError):
+        resize_raster(raster=[array2],
+                      width=500,
+                      height=500)
+
+    with pytest.raises(TypeError):
+        resize_raster(raster=array2,
+                      width=[500],
+                      height=[500])
+
+
+# Testing calculate_difference
+###########################################################
+def test_calculate_difference():
+    from gemgis.raster import calculate_difference
+
+    array_diff = calculate_difference(raster1=np.ones(9).reshape(3, 3),
+                                      raster2=np.zeros(9).reshape(3, 3))
+    assert array_diff.ndim == 2
+    assert array_diff.shape == (3, 3)
+    for i in range(array_diff.shape[1]):
+        for j in range(array_diff.shape[0]):
+            assert array_diff[j][i] == 1
+
+
+# Testing calculate_difference
+###########################################################
+@pytest.mark.parametrize("dem",
+                         [
+                             rasterio.open('../../gemgis/tests/data/raster1.tif')
+                         ])
+def test_calculate_difference(dem):
+    from gemgis.raster import calculate_difference
+
+    array_diff = calculate_difference(raster1=dem.read(1),
+                                      raster2=dem.read(1) + 5,
+                                      flip_array=False)
+    assert array_diff.ndim == 2
+    assert array_diff.shape == (275, 250)
+    for i in range(array_diff.shape[1]):
+        for j in range(array_diff.shape[0]):
+            assert round(array_diff[j][i]) == -5
+
+
+# Testing calculate_difference
+###########################################################
+def test_calculate_difference_error():
+    from gemgis.raster import calculate_difference
+
+    with pytest.raises(TypeError):
+        calculate_difference(raster1=[np.ones(9).reshape(3, 3)],
+                             raster2=np.zeros(9).reshape(3, 3))
+    with pytest.raises(TypeError):
+        calculate_difference(raster1=np.ones(9).reshape(3, 3),
+                             raster2=[np.zeros(9).reshape(3, 3)])
+
+
+# Testing read_msh
+###########################################################
+def test_read_msh():
+    from gemgis.raster import read_msh
+
+    data = read_msh('../../gemgis/tests/data/GM_Breccia.msh')
+
+    assert isinstance(data, dict)
+    assert 'Tri' in data
+    assert 'Location' in data
+    assert isinstance(data['Tri'], np.ndarray)
+    assert isinstance(data['Location'], np.ndarray)
+
+
+# Testing create_filepaths
+###########################################################
+def test_create_filepaths():
+    from gemgis.raster import create_filepaths
+
+    paths = create_filepaths('../../gemgis_data/data/tests/', search_criteria='test_wcs*.tif')
+
+    assert isinstance(paths, list)
+    assert paths == ['../../gemgis_data/data/tests\\test_wcs_raster.tif']
+    assert isinstance(paths[0], str)
+
+
+# Testing create_filepaths
+###########################################################
+def test_create_src_list():
+    from gemgis.raster import create_src_list, create_filepaths
+
+    paths = create_filepaths('../../gemgis_data/data/tests/', search_criteria='test_wcs*.tif')
+    source_paths = create_src_list(dirpath='', search_criteria='', filepaths=paths)
+
+    assert isinstance(paths, list)
+    assert paths == ['../../gemgis_data/data/tests\\test_wcs_raster.tif']
+
+    assert isinstance(source_paths, list)
+    assert isinstance(source_paths[0], rasterio.io.DatasetReader)
+    assert source_paths[0].name == '../../gemgis_data/data/tests\\test_wcs_raster.tif'
+
+
+# Testing merge_tiles
+###########################################################
+def test_merge_tiles():
+    from gemgis.raster import merge_tiles, create_src_list, create_filepaths
+
+    paths = create_filepaths('../../gemgis_data/data/tests/', search_criteria='test_wcs*.tif')
+    source_paths = create_src_list(dirpath='', search_criteria='', filepaths=paths)
+
+    assert isinstance(paths, list)
+    assert paths == ['../../gemgis_data/data/tests\\test_wcs_raster.tif']
+
+    assert isinstance(source_paths, list)
+    assert isinstance(source_paths[0], rasterio.io.DatasetReader)
+    assert source_paths[0].name == '../../gemgis_data/data/tests\\test_wcs_raster.tif'
+
+    mosaic, transform = merge_tiles(src_files=source_paths)
+
+    assert isinstance(mosaic, np.ndarray)
+    assert isinstance(transform, affine.Affine)
+
+
+# Testing save_array_as_tiff
+###########################################################
+@pytest.mark.parametrize("raster",
+                         [
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
+                         ])
+def test_save_raster_as_tiff(raster):
+    from gemgis.raster import save_as_tiff
+
+    save_as_tiff('test', raster, [0, 1069, 0, 972], 'EPSG:4326')
+
+    assert raster.ndim == 2
+    assert raster.shape == (1069, 972)
+    assert isinstance(raster, np.ndarray)
+
+
+@pytest.mark.parametrize("raster",
+                         [
+                             np.load('../../gemgis_data/data/tests/array_rbf.npy')
+                         ])
+def test_save_raster_as_tiff(raster):
+    from gemgis.raster import save_as_tiff
+
+    with pytest.raises(TypeError):
+        save_as_tiff(['test'], raster, [0, 1069, 0, 972], 'EPSG:4326')
+    with pytest.raises(TypeError):
+        save_as_tiff('test', [raster], [0, 1069, 0, 972], 'EPSG:4326')
+    with pytest.raises(TypeError):
+        save_as_tiff('test', raster, (0, 1069, 0, 972), 'EPSG:4326')
+    with pytest.raises(TypeError):
+        save_as_tiff('test', raster, [0, 1069, 0, 972], ['EPSG:4326'])
